
import os
import numpy as np
import geopandas as gpd
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
from matplotlib.patches import Circle
from matplotlib.colors import LinearSegmentedColormap

from energy_demand.plotting import fig_p2_weather_val, result_mapping
from energy_demand.basic import basic_functions
from energy_demand.technologies import tech_related
from energy_demand.read_write import write_data
from energy_demand.basic import conversions

def total_annual_demand(
        df_data_input,
        path_shapefile_input,
        regions,
        pop_data,
        simulation_yr_to_plot,
        result_path,
        fig_name,
        field_to_plot,
        unit='GW'
    ):
    """
    """
    if unit == 'GW':
        conversion_factor = 1
    elif unit == 'kW':
        conversion_factor = conversions.gwh_to_kwh(gwh=1) #GW to KW
    elif unit == 'percentage':
        conversion_factor = 1
    else:
        raise Exception("Not defined unit")

    df_data_input = df_data_input * conversion_factor

    # Load uk shapefile
    uk_shapefile = gpd.read_file(path_shapefile_input)

    # Population of simulation year
    pop_sim_yr = pop_data[simulation_yr_to_plot]

    regions = list(df_data_input.columns)
    nr_of_regions = df_data_input.shape[1]
    nr_of_realisations = df_data_input.shape[0]

    # Mean over all realisations
    mean = df_data_input.mean(axis=0)

    # Mean normalized with population
    mean_norm_pop = df_data_input.mean(axis=0) / pop_sim_yr

    # Standard deviation over all realisations
    std_dev = df_data_input.std(axis=0)

    max_entry = df_data_input.max(axis=0) #maximum entry for every hour
    min_entry = df_data_input.min(axis=0) #maximum entry for every hour

    print("---- Calculate average per person")
    tot_person = sum(pop_sim_yr)
    print(df_data_input.iloc[0])
    tot_demand = sum(df_data_input.iloc[0])
    print("TOT PERSON: " + str(tot_person))
    print("TOT PERSON: " + str(tot_demand))
    print('AVERAGE KW per Person " '+ str(tot_demand / tot_person))

    print(df_data_input)
    regional_statistics_columns = [
        'name',
        'mean',
        'mean_norm_pop',
        'std_dev']#
        #'diff_av_max',
        #'mean_pp',
        #'diff_av_max_pp',
        #'std_dev_average_every_h',
        #'std_dev_peak_h_norm_pop']

    df_stats = pd.DataFrame(columns=regional_statistics_columns)

    for region_name in regions:

        line_entry = [[
            str(region_name),
            mean[region_name],
            mean_norm_pop[region_name],
            std_dev[region_name]
            #diff_av_max,
            #mean_peak_h_pp,
            #diff_av_max_pp,
            #std_dev_average_every_h,
            #std_dev_peak_h_norm_pop
            ]]

        line_df = pd.DataFrame(
            line_entry,
            columns=regional_statistics_columns)

        df_stats = df_stats.append(line_df)

    # ---------------
    # Create spatial maps
    # http://darribas.org/gds15/content/labs/lab_03.html
    # http://nbviewer.jupyter.org/gist/jorisvandenbossche/57d392c085901eb4981054402b37b6b1
    # ---------------
    # Merge stats to geopanda
    shp_gdp_merged = uk_shapefile.merge(df_stats, on='name')

    # Assign projection
    crs = {'init': 'epsg:27700'} #27700: OSGB_1936_British_National_Grid
    uk_gdf = gpd.GeoDataFrame(shp_gdp_merged, crs=crs)
    ax = uk_gdf.plot()

    # Assign bin colors according to defined cmap and whether
    # plot with min_max values or only min/max values
    #bin_values = [0, 0.0025, 0.005, 0.0075, 0.01]
    nr_of_intervals = 6

    bin_values = result_mapping.get_reasonable_bin_values_II(
        data_to_plot=list(uk_gdf[field_to_plot]),
        nr_of_intervals=nr_of_intervals)

    #print(list(uk_gdf[field_to_plot]))
    print("field_to_plot: {} BINS: {}".format(field_to_plot, bin_values))

    uk_gdf, cmap_rgb_colors, color_zero, min_value, max_value = fig_p2_weather_val.user_defined_bin_classification(
        uk_gdf,
        field_to_plot, 
        bin_values=bin_values)

    # plot with face color attribute
    uk_gdf.plot(ax=ax, facecolor=uk_gdf['bin_color'], edgecolor='black', linewidth=0.5)

    # TODO IMRPVE: MAKE CORRECT ONE FOR NEW PROCESSING
    legend_handles = result_mapping.get_legend_handles(
        bin_values[1:-1],
        cmap_rgb_colors,
        color_zero,
        min_value,
        max_value)

    legend = plt.legend(
        handles=legend_handles,
        title="Unit: {} field: {}".format(unit, field_to_plot),
        prop={'size': 8},
        loc='upper center',
        bbox_to_anchor=(0.5, -0.05),
        frameon=False)

    # Remove coordinates from figure
    ax.set_yticklabels([])
    ax.set_xticklabels([])

    legend.get_title().set_fontsize(8)

    # PLot bins on plot
    '''plt.text(
        0,
        -20,
        bin_values[:-1],
        fontsize=8)'''

    # --------
    # Labeling
    # --------
    plt.title("Peak demand over time")

    plt.tight_layout()
    #plt.show()

    plt.savefig(os.path.join(result_path, fig_name))
    plt.close()




"""
"""
import os
import pandas as pd
import matplotlib.pyplot as plt
import copy
import pylab

from energy_demand.plotting import basic_plot_functions
from energy_demand.basic import conversions
from energy_demand.plotting import plotting_styles
from energy_demand.read_write import write_data

colors = {

    # High elec
    'h_min': '#004529',
    'h_c': '#238443',
    'h_max': '#78c679',

    # Low elec
    'l_min': '#800026',
    'l_c': '#e31a1c',
    'l_max': '#fd8d3c',

    'other1': '#C044FF',
    'other2': '#3DF735',
    'other3': '#AD6D70',
    'other4': '#EC2504',
    'other5': '#8C0B90'}

marker_list = plotting_styles.marker_list()

marker_styles = {
    'h_max': marker_list[0],
    'h_min': marker_list[1],
    'l_min': marker_list[2],
    'l_max': marker_list[6],
}

def scenario_over_time(
        scenario_result_container,
        sim_yrs,
        fig_name,
        result_path,
        plot_points,
        crit_smooth_line=True,
        seperate_legend=False
    ):
    """Plot peak over time
    """
    statistics_to_print = []

    fig = plt.figure(
        figsize=basic_plot_functions.cm2inch(10, 10)) #width, height

    ax = fig.add_subplot(1, 1, 1)

    for cnt_scenario, i in enumerate(scenario_result_container):
        scenario_name = i['scenario_name']
        national_peak = i['national_peak']

        # dataframe with national peak (columns= simulation year, row: Realisation) 

        # Calculate quantiles
        quantile_95 = 0.95
        quantile_05 = 0.05

        try:
            color = colors[scenario_name]
            marker = marker_styles[scenario_name]
        except KeyError:
            color = list(colors.values())[cnt_scenario]

        try:
            marker = marker_styles[scenario_name]
        except KeyError:
            marker = list(marker_styles.values())[cnt_scenario]

        print("SCENARIO NAME {}  {}".format(scenario_name, color))

        # Calculate average across all weather scenarios
        mean_national_peak = national_peak.mean(axis=0)
        mean_national_peak_sim_yrs = copy.copy(mean_national_peak)

        statistics_to_print.append("scenario: {} values over years: {}".format(scenario_name, mean_national_peak_sim_yrs))

        # Standard deviation over all realisations
        df_q_05 = national_peak.quantile(quantile_05)
        df_q_95 = national_peak.quantile(quantile_95)

        statistics_to_print.append("scenario: {} df_q_05: {}".format(scenario_name, df_q_05))
        statistics_to_print.append("scenario: {} df_q_95: {}".format(scenario_name, df_q_95))

        # --------------------
        # Try to smooth lines
        # --------------------
        sim_yrs_smoothed = sim_yrs
        if crit_smooth_line:
            try:
                sim_yrs_smoothed, mean_national_peak_smoothed = basic_plot_functions.smooth_data(sim_yrs, mean_national_peak, num=40000)
                _, df_q_05_smoothed = basic_plot_functions.smooth_data(sim_yrs, df_q_05, num=40000)
                _, df_q_95_smoothed = basic_plot_functions.smooth_data(sim_yrs, df_q_95, num=40000)

                mean_national_peak = pd.Series(mean_national_peak_smoothed, sim_yrs_smoothed)
                df_q_05 = pd.Series(df_q_05_smoothed, sim_yrs_smoothed)
                df_q_95 = pd.Series(df_q_95_smoothed, sim_yrs_smoothed)
            except:
                sim_yrs_smoothed = sim_yrs

        # -----------------------
        # Plot lines
        # ------------------------
        plt.plot(
            mean_national_peak,
            label="{} (mean)".format(scenario_name),
            color=color)

        # ------------------------
        # Plot markers
        # ------------------------
        if plot_points:

            plt.scatter(
                sim_yrs,
                mean_national_peak_sim_yrs,
                c=color,
                marker=marker,
                edgecolor='black',
                linewidth=0.5,
                s=15,
                clip_on=False) #do not clip points on axis


        # Plottin qunatilse and average scenario
        df_q_05.plot.line(color=color, linestyle='--', linewidth=0.1, label='_nolegend_') #, label="0.05")
        df_q_95.plot.line(color=color, linestyle='--', linewidth=0.1, label='_nolegend_') #, label="0.05")

        plt.fill_between(
            sim_yrs_smoothed,
            list(df_q_95),  #y1
            list(df_q_05),  #y2
            alpha=0.25,
            facecolor=color,
            )

    plt.xlim(2015, 2050)
    plt.ylim(0)

    # --------
    # Different style
    # --------
    ax = plt.gca()
    ax.grid(which='major', color='black', axis='y', linestyle='--')
    
    ax.spines['right'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.spines['top'].set_visible(False)

    plt.tick_params(
        axis='y',          # changes apply to the x-axis
        which='both',      # both major and minor ticks are affected
        bottom=False,      # ticks along the bottom edge are off
        top=False,         # ticks along the top edge are off
        left=False,
        right=False,
        labelbottom=False,
        labeltop=False,
        labelleft=True,
        labelright=False) # labels along the bottom edge are off

    # --------
    # Legend
    # --------
    legend = plt.legend(
        #title="tt",
        ncol=2,
        prop={'size': 10},
        loc='upper center',
        bbox_to_anchor=(0.5, -0.1),
        frameon=False)
    legend.get_title().set_fontsize(8)

    if seperate_legend:
        basic_plot_functions.export_legend(
            legend,
            os.path.join(result_path, "{}__legend.pdf".format(fig_name)))
        legend.remove()

    # --------
    # Labeling
    # --------
    plt.ylabel("national peak demand (GW)")
    #plt.xlabel("year")
    #plt.title("Title")

    plt.tight_layout()

    plt.savefig(os.path.join(result_path, fig_name))
    plt.close()

    # Write info to txt
    write_data.write_list_to_txt(
        os.path.join(result_path, fig_name).replace(".pdf", ".txt"),
        statistics_to_print)

def fueltypes_over_time(
        scenario_result_container,
        sim_yrs,
        fig_name,
        fueltypes,
        result_path,
        plot_points=False,
        unit='TWh',
        crit_smooth_line=True,
        seperate_legend=False
    ):
    """Plot fueltypes over time
    """
    statistics_to_print = []

    fig = plt.figure(
        figsize=basic_plot_functions.cm2inch(10, 10)) #width, height
    ax = fig.add_subplot(1, 1, 1)

    colors = {
        # Low elec
        'electricity':  '#3e3838',
        'gas':          '#ae7c7c',
        'hydrogen':     '#6cbbb3',
    }

    line_styles_default = plotting_styles.linestyles()

    linestyles = {
        'h_max': line_styles_default[0],
        'h_min': line_styles_default[6],
        'l_min': line_styles_default[8],
        'l_max': line_styles_default[9],
    }

    for cnt_scenario, i in enumerate(scenario_result_container):
        scenario_name = i['scenario_name']

        for cnt_linestyle, fueltype_str in enumerate(fueltypes):
            national_sum = i['national_{}'.format(fueltype_str)]

            if unit == 'TWh':
                unit_factor = conversions.gwh_to_twh(1)
            elif unit == 'GWh':
                unit_factor = 1
            else:
                raise Exception("Wrong unit")

            national_sum = national_sum * unit_factor

            # Calculate quantiles
            quantile_95 = 0.95
            quantile_05 = 0.05

            try:
                color = colors[fueltype_str]
            except KeyError:
                #color = list(colors.values())[cnt_linestyle]
                raise Exception("Wrong color")

            try:
                linestyle = linestyles[scenario_name]
            except KeyError:
                linestyle = list(linestyles.values())[cnt_scenario]
            try:
                marker = marker_styles[scenario_name]
            except KeyError:
                marker = list(marker_styles.values())[cnt_scenario]

            # Calculate average across all weather scenarios
            mean_national_sum = national_sum.mean(axis=0)
            mean_national_sum_sim_yrs = copy.copy(mean_national_sum)

            statistics_to_print.append("{} fueltype_str: {} mean_national_sum_sim_yrs: {}".format(scenario_name, fueltype_str, mean_national_sum_sim_yrs))

            # Standard deviation over all realisations
            df_q_05 = national_sum.quantile(quantile_05)
            df_q_95 = national_sum.quantile(quantile_95)

            statistics_to_print.append("{} fueltype_str: {} df_q_05: {}".format(scenario_name, fueltype_str, df_q_05))
            statistics_to_print.append("{} fueltype_str: {} df_q_95: {}".format(scenario_name, fueltype_str, df_q_95))
            # --------------------
            # Try to smooth lines
            # --------------------
            sim_yrs_smoothed = sim_yrs
            if crit_smooth_line:
                try:
                    sim_yrs_smoothed, mean_national_sum_smoothed = basic_plot_functions.smooth_data(sim_yrs, mean_national_sum, num=500)
                    _, df_q_05_smoothed = basic_plot_functions.smooth_data(sim_yrs, df_q_05, num=500)
                    _, df_q_95_smoothed = basic_plot_functions.smooth_data(sim_yrs, df_q_95, num=500)

                    mean_national_sum = pd.Series(mean_national_sum_smoothed, sim_yrs_smoothed)
                    df_q_05 = pd.Series(df_q_05_smoothed, sim_yrs_smoothed)
                    df_q_95 = pd.Series(df_q_95_smoothed, sim_yrs_smoothed)
                except:
                    print("did not owrk {} {}".format(fueltype_str, scenario_name))
                    pass
            
            # ------------------------
            # Plot lines
            # ------------------------
            plt.plot(
                mean_national_sum,
                label="{} {}".format(fueltype_str, scenario_name),
                linestyle=linestyle,
                color=color,
                zorder=1,
                clip_on=True)

            # ------------------------
            # Plot markers
            # ------------------------
            if plot_points:
                plt.scatter(
                    sim_yrs,
                    mean_national_sum_sim_yrs,
                    marker=marker,
                    edgecolor='black',
                    linewidth=0.5,
                    c=color,
                    zorder=2,
                    s=15,
                    clip_on=False) #do not clip points on axis

            # Plottin qunatilse and average scenario
            df_q_05.plot.line(color=color, linestyle='--', linewidth=0.1, label='_nolegend_') #, label="0.05")
            df_q_95.plot.line(color=color, linestyle='--', linewidth=0.1, label='_nolegend_') #, label="0.05")

            plt.fill_between(
                sim_yrs_smoothed,
                list(df_q_95),  #y1
                list(df_q_05),  #y2
                alpha=0.25,
                facecolor=color,
                )

    plt.xlim(2015, 2050)
    plt.ylim(0)

    ax = plt.gca()

    # Major ticks every 20, minor ticks every 5
    major_ticks = [200, 400, 600] #np.arange(0, 600, 200)
    minor_ticks = [100, 200, 300, 400, 500, 600] #np.arange(0, 600, 100)

    ax.set_yticks(major_ticks)
    ax.set_yticks(minor_ticks, minor=True)

    # And a corresponding grid
    ax.grid(
        which='both',
        color='black',
        linewidth=0.5,
        axis='y',
        linestyle=line_styles_default[3]) #[6])

    # Or if you want different settings for the grids:
    ax.grid(which='minor', axis='y', alpha=0.4)
    ax.grid(which='major', axis='y', alpha=0.8)

    # Achsen
    ax.spines['right'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.spines['top'].set_visible(False)

    # Ticks
    plt.tick_params(
        axis='y',          # changes apply to the x-axis
        which='both',      # both major and minor ticks are affected
        bottom=False,      # ticks along the bottom edge are off
        top=False,         # ticks along the top edge are off
        left=False,
        right=False,
        labelbottom=False,
        labeltop=False,
        labelleft=True,
        labelright=False) # labels along the bottom edge are off

    # --------
    # Legend
    # --------
    legend = plt.legend(
        #title="tt",
        ncol=2,
        prop={'size': 6},
        loc='upper center',
        bbox_to_anchor=(0.5, -0.1),
        frameon=False)
    legend.get_title().set_fontsize(8)

    if seperate_legend:
        basic_plot_functions.export_legend(
            legend,
            os.path.join(result_path, "{}__legend.pdf".format(fig_name)))
        legend.remove()

    # --------
    # Labeling
    # --------
    plt.ylabel("national fuel over time [in {}]".format(unit))
    #plt.xlabel("year")
    #plt.title("Title")

    plt.tight_layout()
    #plt.show()
    plt.savefig(os.path.join(result_path, fig_name))
    plt.close()

    write_data.write_list_to_txt(
        os.path.join(result_path, fig_name).replace(".pdf", ".txt"),
        statistics_to_print)



"""Read in model results and plot results
"""
import os
import pandas as pd

from energy_demand.read_write import data_loader, read_data
from energy_demand.basic import date_prop
from energy_demand.basic import basic_functions
from energy_demand.basic import lookup_tables
from energy_demand.read_write import read_weather_results
from energy_demand.plotting import fig_3_weather_map
from energy_demand.technologies import tech_related
from energy_demand.plotting import fig_3_plot_over_time

def main(
        scenarios_path,
        path_shapefile_input,
        base_yr,
        simulation_yr_to_plot=2050
    ):
    """Read in all results and plot PDFs

    Arguments
    ----------
    scenarios_path : str
        Path to results
    path_shapefile_input : str
        Path to shapefile
    plot_crit_dict : dict
        Criteria to select plots to plot
    base_yr : int
        Base year
    comparison_year : int
        Year to generate comparison plots
    """
    print("...Start creating plots")
    data = {}

    # ---------------------------------------------------------
    # Iterate folders and read out all weather years and stations
    # ---------------------------------------------------------
    to_ignores = [
        'model_run_pop',
        'PDF_validation',
        '_results_weather_plots']

    endings_to_ignore = [
        '.pdf',
        '.txt',
        '.ini']

    # -------------------
    # Create result folder
    # -------------------
    result_path = os.path.join(scenarios_path, '_results_weather_plots')
    basic_functions.del_previous_setup(result_path)

    all_scenarios = os.listdir(scenarios_path)
    scenario_result_container = []
    basic_functions.create_folder(result_path)

    for scenario_nr, scenario_name in enumerate(all_scenarios):
        print(" ")
        print("Scenario: {}".format(scenario_name))
        print(" ")
        scenario_path = os.path.join(scenarios_path, scenario_name)
        all_result_folders = os.listdir(scenario_path)
        
        paths_folders_result = []

        for result_folder in all_result_folders:
            if result_folder not in to_ignores and result_folder[-4:] not in endings_to_ignore:
                paths_folders_result.append(
                    os.path.join(scenario_path, result_folder))

        fueltype_str_to_create_maps = ['electricity']

        fueltype_str ='electricity'
        fueltype_int = tech_related.get_fueltype_int(fueltype_str)

        ####################################################################
        # Collect regional simulation data for every realisation
        ####################################################################
        total_regional_demand_electricity = pd.DataFrame()
        peak_hour_demand = pd.DataFrame()
        national_peak = pd.DataFrame()
        regional_share_national_peak = pd.DataFrame()
        national_electricity = pd.DataFrame()
        national_gas = pd.DataFrame()
        national_hydrogen = pd.DataFrame()

        for path_result_folder in paths_folders_result:
            print("path_result_folder: " + str(path_result_folder))

            data = {}

            # Simulation information is read in from .ini file for results
            data['enduses'], data['assumptions'], data['regions'] = data_loader.load_ini_param(
                os.path.join(path_result_folder))

            pop_data = read_data.read_scenaric_population_data(
                os.path.join(path_result_folder, 'model_run_pop'))
  

            path_result_folder = os.path.join(path_result_folder, 'simulation_results')
            path_result_folder_model_runs = os.path.join(path_result_folder, 'model_run_results_txt')

            data['lookups'] = lookup_tables.basic_lookups()

            # Other information is read in
            data['assumptions']['seasons'] = date_prop.get_season(year_to_model=2015)
            data['assumptions']['model_yeardays_daytype'], data['assumptions']['yeardays_month'], data['assumptions']['yeardays_month_days'] = date_prop.get_yeardays_daytype(year_to_model=2015)

            # --------------------------------------------
            # Reading in results from different model runs
            # --------------------------------------------
            results_container = read_weather_results.read_in_weather_results(
                path_result_folder_model_runs,
                data['assumptions']['seasons'],
                data['assumptions']['model_yeardays_daytype'],
                fueltype_str='electricity')

            # --Total demand (dataframe with row: realisation, column=region)
            realisation_data = pd.DataFrame(
                [results_container['ed_reg_tot_y'][simulation_yr_to_plot][fueltype_int]],
                columns=data['regions'])
            total_regional_demand_electricity = total_regional_demand_electricity.append(realisation_data)

            # National per fueltype electricity
            fueltype_elec_int = tech_related.get_fueltype_int('electricity')
            simulation_yrs_result = [results_container['national_all_fueltypes'][year][fueltype_elec_int] for year in results_container['national_all_fueltypes'].keys()]

            realisation_data = pd.DataFrame(
                [simulation_yrs_result],
                columns=data['assumptions']['sim_yrs'])
            national_electricity = national_electricity.append(realisation_data)

            # National per fueltype gas
            fueltype_elec_int = tech_related.get_fueltype_int('gas')
            simulation_yrs_result = [results_container['national_all_fueltypes'][year][fueltype_elec_int] for year in results_container['national_all_fueltypes'].keys()]

            realisation_data = pd.DataFrame(
                [simulation_yrs_result],
                columns=data['assumptions']['sim_yrs'])
            national_gas = national_gas.append(realisation_data)

            # National per fueltype hydrogen
            fueltype_elec_int = tech_related.get_fueltype_int('hydrogen')
            simulation_yrs_result = [results_container['national_all_fueltypes'][year][fueltype_elec_int] for year in results_container['national_all_fueltypes'].keys()]

            realisation_data = pd.DataFrame(
                [simulation_yrs_result],
                columns=data['assumptions']['sim_yrs'])
            national_hydrogen = national_hydrogen.append(realisation_data)

            # --Peak day demand (dataframe with row: realisation, column=region)
            realisation_data = pd.DataFrame(
                [results_container['ed_reg_peakday_peak_hour'][simulation_yr_to_plot][fueltype_int]],
                columns=data['regions'])

            peak_hour_demand = peak_hour_demand.append(realisation_data)

            # --National peak
            simulation_yrs_result = [results_container['national_peak'][year][fueltype_int] for year in results_container['national_peak'].keys()]

            realisation_data = pd.DataFrame(
                [simulation_yrs_result],
                columns=data['assumptions']['sim_yrs'])
            national_peak = national_peak.append(realisation_data)

            # --Regional percentage of national peak demand
            realisation_data = pd.DataFrame(
                [results_container['regional_share_national_peak'][simulation_yr_to_plot]],
                columns=data['regions'])

            regional_share_national_peak = regional_share_national_peak.append(realisation_data)

        # Add to scenario container
        scenario_result_container.append({
            'scenario_name': scenario_name,
            'peak_hour_demand': peak_hour_demand,
            'national_peak': national_peak,
            'regional_share_national_peak': regional_share_national_peak,

            'total_regional_demand_electricity': total_regional_demand_electricity,
            'national_electricity': national_electricity,
            'national_gas': national_gas,
            'national_hydrogen': national_hydrogen,
        })

    # ------------------------------
    # Plot national sum over time per fueltype and scenario
    # ------------------------------
    print("... plotting national sum of fueltype over time")
    fig_3_plot_over_time.fueltypes_over_time(
        scenario_result_container=scenario_result_container,
        sim_yrs=data['assumptions']['sim_yrs'],
        fig_name="fueltypes_over_time_{}.pdf".format(fueltype_str),
        fueltypes=['electricity', 'gas', 'hydrogen'],
        result_path=result_path,
        unit='TWh',
        plot_points=True,
        crit_smooth_line=True,
        seperate_legend=False)

    # ------------------------------
    # Plot national peak change over time for each scenario
    # including weather variability
    # ------------------------------
    fig_3_plot_over_time.scenario_over_time(
        scenario_result_container=scenario_result_container,
        sim_yrs=data['assumptions']['sim_yrs'],
        fig_name="scenarios_peak_over_time_{}.pdf".format(fueltype_str),
        plot_points=True,
        result_path=result_path,
        crit_smooth_line=True,
        seperate_legend=False)

    # ------------------------------
    # Plotting spatial results for electricity
    # ------------------------------
    for i in scenario_result_container:
        scenario_name = i['scenario_name']
        total_regional_demand_electricity = i['total_regional_demand_electricity']
        peak_hour_demand = i['peak_hour_demand']
        regional_share_national_peak = i['regional_share_national_peak']

        print("... plot spatial map of total annual demand")
        field_to_plot = 'std_dev'
        fig_3_weather_map.total_annual_demand(
            total_regional_demand_electricity,
            path_shapefile_input,
            data['regions'],
            pop_data=pop_data,
            simulation_yr_to_plot=simulation_yr_to_plot,
            result_path=result_path,
            fig_name="{}__tot_demand__{}_{}_{}.pdf".format(scenario_name, field_to_plot, fueltype_str, simulation_yr_to_plot),
            field_to_plot=field_to_plot,
            unit='GW')

        print("... plot spatial map of peak hour demand")
        field_to_plot = 'std_dev'
        fig_3_weather_map.total_annual_demand(
            peak_hour_demand,
            path_shapefile_input,
            data['regions'],
            pop_data=pop_data,
            simulation_yr_to_plot=simulation_yr_to_plot,
            result_path=result_path,
            fig_name="{}__peak_h_demand_{}_{}_{}.pdf".format(scenario_name, field_to_plot, fueltype_str, simulation_yr_to_plot),
            field_to_plot=field_to_plot,
            unit='GW')

        print("... plot spatial map of percentage of regional peak hour demand")
        field_to_plot = 'mean'
        fig_3_weather_map.total_annual_demand(
            regional_share_national_peak,
            path_shapefile_input,
            data['regions'],
            pop_data=pop_data,
            simulation_yr_to_plot=simulation_yr_to_plot,
            result_path=result_path,
            fig_name="{}__regional_share_national_peak_{}_{}_{}.pdf".format(scenario_name, field_to_plot, fueltype_str, simulation_yr_to_plot),
            field_to_plot=field_to_plot,
            unit='percentage')

        field_to_plot = 'std_dev'
        fig_3_weather_map.total_annual_demand(
            regional_share_national_peak,
            path_shapefile_input,
            data['regions'],
            pop_data=pop_data,
            simulation_yr_to_plot=simulation_yr_to_plot,
            result_path=result_path,
            fig_name="{}__regional_share_national_peak_{}_{}_{}.pdf".format(scenario_name, field_to_plot, fueltype_str, simulation_yr_to_plot),
            field_to_plot=field_to_plot,
            unit='percentage')

    print("===================================")
    print("... finished reading and plotting results")
    print("===================================")


path_shapefile_input="C:/Users/cenv0553/ED/data/energy_demand/region_definitions/lad_2016_uk_simplified.shp"

main(
    scenarios_path="C:/_FF",
    path_shapefile_input=path_shapefile_input,
    base_yr=2015)



"""Load paramter values from csv files
"""
import os
import pandas as pd
from energy_demand.read_write import read_data
from collections import defaultdict

def load_full_parameters(
        temp_path,
        parameter_names
    ):
    """Load all csv files
    """
    container_out = {}

    for name in parameter_names:

        container = defaultdict(dict)

        path_file = os.path.join(temp_path, "{}.csv".format(name))
        #print("path_file " + str(path_file))

        # Load files with headers {"region", "year", "value", "interval"}
        #df_param = pd.read_csv(path_file, index_col=False)

        #list_with_dict_entries = df_param.to_dict(orient='region') #Convert to dict

        #for entry in list_with_dict_entries:
        #    region = entry['region']
        #    value = entry['value']
        #    year = entry['year']
        #
        #    container[region][year] = value
        #
        #container_out[name] = dict(container)

    return container



"""Region Class
"""
import numpy as np

from energy_demand.geography import weather_station_location

class Region(object):
    """Region class

    Arguments
    ---------
    name : str
        Name of region
    weather_region_id : str
        ID of closest weather station
    longitude : float
        Longitude coordinate
    latitude : float
        Latitude coordinate
    fuel_disagg : dict
        Nested dict by region, enduse => np.array, single dimension for fuel type
    weather_stations : dict
        Weather stations of current weather year
    weather_stations_by : dict
        Weather stations of base weather year
    weather_reg_cy : dict
        Weather station region objects of current weather year
    weather_reg_by : dict
        Weather station region objects of current weather year

    Note
    ----
    *   The closest weather station is calculated
    """
    def __init__(
            self,
            name,
            longitude,
            latitude,
            region_fuel_disagg,
            weather_reg_cy,
            weather_reg_by
        ):
        """Constructor
        """
        self.name = name
        self.longitude = longitude
        self.latitude = latitude
        self.fuels = region_fuel_disagg

        # =================
        # Calculate Weather Correction factor in relation to base year
        #
        # Because the number of HDD and CDD may be different for different
        # weather years, a linear relationship between energy demand
        # related to heating and cooling is assumed and a weather
        # correction factor is calculated based on HDD and CDD calculations
        # for the weather base year and the actual weather data depending
        # on the closest weather region
        # =================

        # Climate correction factor (hdd)

        # Residential
        factor = np.sum(weather_reg_cy.rs_hdd_by) / np.sum(weather_reg_by.rs_hdd_by)
        if np.isnan(factor):
            f_climate_hdd_rs = 1
        else:
            f_climate_hdd_rs = factor

        # Service
        factor = np.sum(weather_reg_cy.ss_hdd_by) / np.sum(weather_reg_by.ss_hdd_by)
        if np.isnan(factor):
            f_climate_hdd_ss = 1
        else:
            f_climate_hdd_ss = factor

        # Industry
        factor = np.sum(weather_reg_cy.is_hdd_by) / np.sum(weather_reg_by.is_hdd_by)

        if np.isnan(factor):
            f_climate_hdd_is = 1
        else:
            f_climate_hdd_is = factor

        # Climate correction factor (cdd)
        factor = np.sum(weather_reg_cy.ss_cdd_by) / np.sum(weather_reg_by.ss_cdd_by)
        #f_climate_cdd_is = np.sum(weather_reg_cy.is_cdd_by) / np.sum(weather_reg_by.is_cdd_by)
        if np.isnan(factor):
            f_climate_cdd_ss = 1
        else:
            f_climate_cdd_ss = factor

        self.f_weather_correction = {
            'residential': {'hdd': f_climate_hdd_rs, 'cdd': None},
            'service': {'hdd': f_climate_hdd_ss, 'cdd': f_climate_cdd_ss},
            'industry': {'hdd': f_climate_hdd_is, 'cdd': None}}

        # Visual testing purposes
        #from energy_demand.read_write import data_loader
        #data_loader.print_closest_and_region(
        #    weather_stations,
        #    {name: {'latitude': latitude, 'longitude': longitude}},
        #    {self.closest_weather_reg: weather_stations[self.closest_weather_reg]})



"""Generate YAML files with standard yaml 
"""
import os
import configparser
from energy_demand.assumptions import general_assumptions
from energy_demand.assumptions import strategy_vars_def
from energy_demand.read_write import data_loader
from energy_demand.basic import lookup_tables

# ----------------------------
# Configuration
# ----------------------------
path_config = os.path.abspath(
    os.path.join(
            os.path.dirname(__file__), '..', '..', '..', "config"))

path_main = os.path.abspath(
    os.path.join(
            os.path.dirname(__file__), '..', "config_data"))
# Configuration paths
config = configparser.ConfigParser()
config.read(os.path.join(path_config, 'wrapperconfig.ini'))

# --------
# Other configuration
# --------
base_yr = 2015
sim_yrs = [2015, 2050]

paths = data_loader.load_paths(path_main)
data_path = os.path.normpath(config['PATHS']['path_local_data'])

local_paths = data_loader.get_local_paths(
    data_path)

data = {}

data['enduses'], data['sectors'], data['fuels'] = data_loader.load_fuels(
    paths)

assumptions = general_assumptions.Assumptions(
    base_yr=user_defined_base_yr,
    curr_yr=2015,
    sim_yrs=sim_yrs,
    paths=paths,
    enduses=data['enduses'],
    sectors=data['sectors'])

# Write parameters to YAML file
_, _ = strategy_vars_def.load_param_assump(
    assumptions)

print("==========")
print("Finishged generating YAML file with scenario parameters")
print("==========")
